"""cubicweb-vtimeline views/forms/actions/components for web ui"""
import json
from itertools import chain

from logilab.common.date import ustrftime

from cubicweb.utils import json_dumps
from cubicweb.predicates import is_instance

try:
    from cubes.vtimeline.views import VeriteCoTimelineJsonDataView, VeriteCoTimelineView
except ImportError:
    pass
else:
    class SubjectVeriteCoTimelineView(VeriteCoTimelineView):
        __select__ = is_instance('Subject')

        def js_story_factory(self, settings):
            return u'''$("a[href$=\'#entity-timeline-tab\']").on('shown.bs.tab', function() {
            if (cw.cubes.brainomics.timelineLoaded === undefined) {
                createStoryJS(%s);
                cw.cubes.brainomics.timelineLoaded = true;}});
            ''' % json_dumps(settings)


    class VeriteCoTimelineJsonDataView(VeriteCoTimelineJsonDataView):
        __select__ = is_instance('Subject')

        def call(self):
            dates = []
            # Date format is %Y,%m,%d for vtimeline sake
            d = {'timeline': {'headline': '', 'type': 'default', 'text': '', 'date': dates}}
            for entity in self.cw_rset.entities():
                # Subject dates
                for admission in entity.reverse_admission_of:
                    study = admission.admission_in[0]
                    for date, label in ((admission.admission_date, self._cw._('Admission in')),
                                        (admission.admission_end_date, self._cw._('Out of'))):
                        if not date:
                            continue
                        text = u'<p>%s - %s</p>' % (label, study.view('incontext'))
                        dates.append({'startDate': ustrftime(date, '%Y,%m,%d'),
                                      'headline': label,
                                      'text': text})
                    # Steps
                    for step in admission.reverse_step_of:
                        if step.step_date:
                            dates.append({'startDate': ustrftime(step.step_date, '%Y,%m,%d'),
                                          'headline': step.name,
                                          'text': u'<p>%s</p>' % study.view('incontext')})
                # Measures
                for measure in entity.reverse_concerns:
                    calendarable = measure.cw_adapt_to('ICalendarable')
                    # Measure adapter
                    if calendarable and calendarable.start:
                        dates.append({'startDate': ustrftime(calendarable.start, '%Y,%m,%d'),
                                      'headline': measure.view('incontext')})
                    # Fallback to assessment
                    elif measure.reverse_generates:
                        calendarable = measure.reverse_generates[0].cw_adapt_to('ICalendarable')
                        if calendarable and calendarable.start:
                            dates.append({'startDate': ustrftime(calendarable.start, '%Y,%m,%d'),
                                          'headline': measure.view('incontext')})
                # Diagnostics/Therapies
                for e in chain(entity.related_diagnostics, entity.related_therapies):
                    calendarable = e.cw_adapt_to('ICalendarable')
                    # Measure adapter
                    if calendarable and calendarable.start:
                        dates.append({'startDate': ustrftime(calendarable.start, '%Y,%m,%d'),
                                      'headline': e.view('incontext')})
                    # Drugtake
                    if e.cw_etype == 'Therapy':
                        for drugtake in e.reverse_taken_in_therapy:
                            date = drugtake.start_taking_date
                            if date:
                                dates.append({'startDate': ustrftime(date, '%Y,%m,%d'),
                                              'headline': drugtake.view('incontext')})
                # SurvivalData
                for surv in entity.reverse_survival_of:
                    if surv.lastnews_date:
                        title = []
                        for attr in ('state_at_lastnews', 'deceased', 'relapse_date'):
                            if getattr(surv, attr) is not None:
                                title.append('%s - %s' % (self._cw._(attr), getattr(surv, attr)))
                        dates.append({'startDate': ustrftime(surv.lastnews_date, '%Y,%m,%d'),
                                      'headline': '<br/>'.join(title)})
                # Add additional data
                self.additional_infos(entity, d)
            self.w(json.dumps(d))

        def additional_infos(self, entity, d):
            """ Allow to override this view with specific infos """
            pass


    def registration_callback(vreg):
        vreg.register(VeriteCoTimelineJsonDataView)
        vreg.register_and_replace(SubjectVeriteCoTimelineView, VeriteCoTimelineView)



"""
Backport from CW 3.19 (0509880fec01)
This should be removed once SuiviMP switch to 3.19 (currently 3.18)
"""

from rql import nodes
from cubicweb.web.facet import RQLPathFacet, DateRangeFacet, cleanup_select, _get_var


class AbstractRangeRQLPathFacet(RQLPathFacet):
    """
    The :class:`AbstractRangeRQLPathFacet` is the base class for
    RQLPathFacet-type facets allowing the use of RangeWidgets-like
    widgets (such as (:class:`FacetRangeWidget`,
    class:`DateFacetRangeWidget`) on the parent :class:`RQLPathFacet`
    target attribute.
    """
    __abstract__ = True
    dynamic_facet = False

    def vocabulary(self):
        """return vocabulary for this facet, eg a list of (label,
        value)"""
        select = self.select
        select.save_state()
        try:
            filtered_variable = self.filtered_variable
            cleanup_select(select, filtered_variable)
            varmap, restrvar = self.add_path_to_select()
            if self.label_variable:
                attrvar = varmap[self.label_variable]
            else:
                attrvar = restrvar
            # start RangeRQLPathFacet
            minf = nodes.Function('MIN')
            minf.append(nodes.VariableRef(restrvar))
            select.add_selected(minf)
            maxf = nodes.Function('MAX')
            maxf.append(nodes.VariableRef(restrvar))
            select.add_selected(maxf)
            # add is restriction if necessary
            if filtered_variable.stinfo['typerel'] is None:
                etypes = frozenset(sol[filtered_variable.name] for sol in select.solutions)
                select.add_type_restriction(filtered_variable, etypes)
            # end RangeRQLPathFacet
            try:
                # this line is to be reported in cw 3.19.0
                args = self.cw_rset.args if self.cw_rset else None
                rset = self.rqlexec(select.as_string(), args)
            except Exception:
                self.exception('error while getting vocabulary for %s, rql: %s',
                               self, select.as_string())
                return ()
        finally:
            select.recover()
        # don't call rset_vocabulary on empty result set, it may be an empty
        # *list* (see rqlexec implementation)
        if rset:
            minv, maxv = rset[0]
            return [(unicode(minv), minv), (unicode(maxv), maxv)]
        return []


    def possible_values(self):
        """return a list of possible values (as string since it's used to
        compare to a form value in javascript) for this facet
        """
        return [strval for strval, val in self.vocabulary()]

    def add_rql_restrictions(self):
        infvalue = self.infvalue()
        supvalue = self.supvalue()
        if infvalue is None or supvalue is None: # nothing sent
            return
        varmap, restrvar = self.add_path_to_select(
            skiplabel=True, skipattrfilter=True)
        restrel = None
        for part in self.path:
            if isinstance(part, basestring):
                part = part.split()
            subject, rtype, object = part
            if object == self.filter_variable:
                restrel = rtype
        assert restrel
        # when a value is equal to one of the limit, don't add the restriction,
        # else we filter out NULL values implicitly
        if infvalue != self.infvalue(min=True):
            self._add_restriction(infvalue, '>=', restrvar, restrel)
        if supvalue != self.supvalue(max=True):
            self._add_restriction(supvalue, '<=', restrvar, restrel)

    def _add_restriction(self, value, operator, restrvar, restrel):
        self.select.add_constant_restriction(restrvar,
                                             restrel,
                                             self.formatvalue(value),
                                             self.target_attr_type, operator)

    def add_path_to_select(self, skiplabel=False, skipattrfilter=False):
        varmap = {'X': self.filtered_variable}
        actual_filter_variable = None
        for part in self.path:
            if isinstance(part, basestring):
                part = part.split()
            subject, rtype, object = part
            if skiplabel and object == self.label_variable:
                continue
            if object == self.filter_variable:
                rschema = self._cw.vreg.schema.rschema(rtype)
                if rschema.final:
                    # filter variable is an attribute variable
                    if self.restr_attr is None:
                        self.restr_attr = rtype
                    if self.restr_attr_type is None:
                        attrtypes = set(obj for subj,obj in rschema.rdefs)
                        if len(attrtypes) > 1:
                            raise Exception('ambigous attribute %s, specify attrtype on %s'
                                            % (rtype, self.__class__))
                        self.restr_attr_type = iter(attrtypes).next()
                    if skipattrfilter:
                        actual_filter_variable = subject
                        continue
            subjvar = _get_var(self.select, subject, varmap)
            if rtype == 'eid' and self.dynamic_facet:
                # Keep eid restriction for dynamic variable
                rel = nodes.make_constant_restriction(subjvar, 'eid', object, 'Int')
            else:
                objvar = _get_var(self.select, object, varmap)
                rel = nodes.make_relation(subjvar, rtype, (objvar,),
                                          nodes.VariableRef)
            self.select.add_restriction(rel)
        if self.restr_attr is None:
            self.restr_attr = 'eid'
        if self.restr_attr_type is None:
            self.restr_attr_type = 'Int'
        if actual_filter_variable:
            restrvar = varmap[actual_filter_variable]
        else:
            restrvar = varmap[self.filter_variable]
        return varmap, restrvar


class RangeRQLPathFacet(AbstractRangeRQLPathFacet, RQLPathFacet):
    """
    The :class:`RangeRQLPathFacet` uses the :class:`FacetRangeWidget`
    on the :class:`AbstractRangeRQLPathFacet` target attribute
    """
    pass


class DateRangeRQLPathFacet(AbstractRangeRQLPathFacet, DateRangeFacet):
    """
    The :class:`DateRangeRQLPathFacet` uses the
    :class:`DateFacetRangeWidget` on the
    :class:`AbstractRangeRQLPathFacet` target attribute
    """
    pass



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# copyright 2013 CEA (Saclay, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

from cubicweb.web.action import Action
from cubicweb.predicates import (is_instance, nonempty_rset, anonymous_user,
                                 one_line_rset, match_user_groups, relation_possible)
from cubicweb.web.views.actions import (CopyAction, ModifyAction, DeleteAction,
                                        ManagePermissionsAction, AddRelatedActions)

from cubes.brainomics.views.download import (CSV_DOWNLOADABLE, XCEDE_DOWNLOADABLE,
                                             ZIP_DOWNLOADABLE, JSON_DOWNLOADABLE)


###############################################################################
### ADMIN ACTIONS #############################################################
###############################################################################
class BrainomicsAbstractAdminAction(Action):
    __abstract__ = True
    __select__ = Action.__select__ & nonempty_rset() & match_user_groups('users', 'managers')
    category = 'action-admin'
    icon = 'question-sign'
    title = None    # action title, i18nable


class BrainomicsWikiAdminAction(BrainomicsAbstractAdminAction):
    __regid__ = 'ctx-admin-link-wiki'
    __select__ = BrainomicsAbstractAdminAction.__select__ & relation_possible('wiki')
    title = _('Update wiki')

    def url(self):
        entity = self.cw_rset.get_entity(0, 0)
        card = entity.wiki
        kwargs = {'__redirectpath': entity.rest_path()}
        if not card:
            start_path = 'add/Card'
            kwargs['__linkto'] = 'wiki:%s:object' % entity.eid
        else:
            start_path = card[0].rest_path()
            kwargs['vid'] = 'edition'
        return self._cw.build_url(start_path, **kwargs)


class BrainomicsCommentAdminAction(BrainomicsAbstractAdminAction):
    __regid__ = 'ctx-admin-link-comment'
    __select__ = BrainomicsAbstractAdminAction.__select__ & relation_possible('comments', role='object')
    title = _('Add comment')

    def url(self):
        entity = self.cw_rset.get_entity(0, 0)
        return self._cw.build_url('add/Comment', __linkto='comments:%s:subject' % entity.eid,
                                  __redirectpath=entity.rest_path())

# Add icon for cubicweb admin action
ModifyAction.icon = 'edit'
DeleteAction.icon = 'remove'
ManagePermissionsAction.icon = 'eye-open'


###############################################################################
### DOWNLOAD ACTIONS ##########################################################
###############################################################################
class BrainomicsAbstractDownloadAction(Action):
    __abstract__ = True
    __select__ = Action.__select__ & nonempty_rset() & ~one_line_rset()
    category = 'download-links'


class ScanZipFileBox(BrainomicsAbstractDownloadAction):
    __regid__ = 'scan_zipfile'
    __select__ = BrainomicsAbstractDownloadAction.__select__  & is_instance(*ZIP_DOWNLOADABLE)
    title = _('Download Zip')
    download_vid = 'data-zip'


class CsvFileBox(BrainomicsAbstractDownloadAction):
    __regid__ = 'csvexport'
    __select__ = BrainomicsAbstractDownloadAction.__select__  & is_instance(*CSV_DOWNLOADABLE)
    title = _('Download CSV')
    download_vid = 'csvexport'


class XcedeBox(BrainomicsAbstractDownloadAction):
    __regid__ = 'xcede_file'
    __select__ = BrainomicsAbstractDownloadAction.__select__  & is_instance(*XCEDE_DOWNLOADABLE)
    title = _('Download Xcede')
    download_vid = 'xcede'


class JsonBox(BrainomicsAbstractDownloadAction):
    __regid__ = 'json_file'
    __select__ = BrainomicsAbstractDownloadAction.__select__  & is_instance(*JSON_DOWNLOADABLE)
    title = _('Download Json')
    download_vid = 'ejsonexport'



###############################################################################
### REGISTRATION CALLBACK #####################################################
###############################################################################
def registration_callback(vreg):
    vreg.register_all(globals().values(), __name__, ())
    # Desactivate some actions for now
    vreg.unregister(CopyAction)
    vreg.unregister(AddRelatedActions)



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# copyright 2013 CEA (Saclay, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
from logilab.mtconverter import xml_escape

from cubicweb.selectors import is_instance
from cubicweb.web.views.xmlrss import XMLView, XMLItemView


###############################################################################
### BASE XCEDE VIEW ###########################################################
###############################################################################
class XcedeView(XMLView):
    """Xcede import"""
    __regid__ = 'xcede'
    title = _('xcede export')
    templatable = False
    content_type = 'text/xml'
    item_vid = 'xcede-item'

    def cell_call(self, row, col):
        self.wview(self.item_vid, self.cw_rset, row=row, col=col)

    def call(self):
        """display a list of entities by calling their <item_vid> view"""
        self.w(u'<?xml version="1.0" encoding="%s"?>\n' % self._cw.encoding)
        self.w(u'<XCEDE xmlns:xcede="http://www.xcede.org/xcede.org/xcede-2" '
               'xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">\n')
        for i in xrange(self.cw_rset.rowcount):
            self.cell_call(i, 0)
        self.w(u'</XCEDE>\n')


###############################################################################
### MEDICALEXP ################################################################
###############################################################################
class SubjectXcedeItemView(XMLItemView):
    __select__ = XMLItemView.__select__ & is_instance('Subject')
    __regid__ = 'xcede-item'
    templatable = False

    def entity_call(self, entity):
        """element as an item for an xml feed"""
        entity.complete()
        self.w(u'<subject ID="%s"/>\n' % entity.identifier)
        # Assessments
        rset = self._cw.execute('Any M WHERE S concerned_by M, M is Assessment, S eid %(e)s',
                                {'e': entity.eid})
        self.wview('xcede-item', rset)
        # Scans, Questionnaires
        # XXX Only use it for now if we have less than 10 subjects - time consuming
        if len(self.cw_rset) < 10:
            for etype in ('Scan', 'QuestionnaireRun', 'GenomicMeasure'):
                rset = self._cw.execute('Any M WHERE M concerns S, S eid %%(e)s, '
                                        'M is %s' % etype, {'e': entity.eid})
                if rset:
                    self.wview('xcede-item', rset)


# XXX AbstractMeasureXcedeItemView ?
# XXX Subjectgroup ?
# XXX link assessment -> studies ?

class AssessmentXcedeItemView(XMLItemView):
    __select__ = XMLItemView.__select__ & is_instance('Assessment')
    __regid__ = 'xcede-item'
    templatable = False

    def entity_call(self, entity):
        """element as an item for an xml feed"""
        entity.complete()
        subject = entity.reverse_concerned_by[0]
        # XXX studyID/visitID -> Assessment
        self.w(u'<visit ID="%(id)s" projectID="%(p)s" subjectID="%(s)s"/>\n'
               % {'id':xml_escape(entity.cwuri),
                  'p': xml_escape(subject.related_studies[0].name),
                  's': xml_escape(subject.identifier)})


class StudyXcedeItemView(XMLItemView):
    __select__ = XMLItemView.__select__ & is_instance('Study')
    __regid__ = 'xcede-item'
    templatable = False

    def entity_call(self, entity):
        """element as an item for an xml feed"""
        entity.complete()
        self.w(u'<project ID="%s">\n' % xml_escape(entity.name))
        self.w(u'<projectInfo>\n')
        self.w(u'<subjectGroupList>\n')
        groups = {}
        rset = self._cw.execute('Any GI, SIS WHERE S related_groups G, S related_studies SI, '
                                'SI eid %(e)s, G identifier GI, S identifier SIS', {'e': entity.eid})
        for gi, si in rset:
            groups.setdefault(gi, []).append(si)
        for gi, sis in groups.iteritems():
            self.w(u'<subjectGroup ID="%s">\n' % gi)
            for si in sis:
                self.w(u'<subjectID>%s</subjectID>\n' % si)
            self.w(u'</subjectGroup>\n')
        self.w(u'</subjectGroupList>\n')
        self.w(u'</projectInfo>\n')
        self.w(u'</project>\n')
        # Add subjects
        # XXX Not used for now, time consuming
        ## rset = self._cw.execute('Any S WHERE S related_studies SI, SI eid %(e)s', {'e': entity.eid})
        ## self.wview('xcede-item', rset)


###############################################################################
### QUESTIONNAIRE #############################################################
###############################################################################
class QuestionnaireXcedeItemView(XMLItemView):
    __select__ = XMLItemView.__select__ & is_instance('Questionnaire')
    __regid__ = 'xcede-item'
    templatable = False

    def entity_call(self, entity):
        """element as an item for an xml feed"""
        entity.complete()
        # XXX studyID/visitID -> Assessment
        self.w(u'<xcede:step ID="%(id)s" name="%(la)s" minOccurences="1" maxOccurences="1" required="true">'
               % {'id':xml_escape(entity.identifier), 'la': xml_escape(entity.name)})
        # Questions/answers
        rset = self._cw.execute('Any Q, QI, QP, QT, QTY, QPA ORDERBY QP ASC '
                                'WHERE QQ is Questionnaire, QQ eid %(e)s, '
                                'Q questionnaire QQ, Q identifier QI, Q position QP, Q text QT, '
                                'Q type QTY, Q possible_answers QPA',
                                {'e': entity.eid})
        self.w(u'<xcede:items>')
        self.wview('xcede-item', rset)
        self.w(u'</xcede:items>')
        self.w(u'</xcede:step>')


class QuestionXcedeItemView(XMLItemView):
    __select__ = XMLItemView.__select__ & is_instance('Question')
    __regid__ = 'xcede-item'
    templatable = False

    def entity_call(self, entity):
        """element as an item for an xml feed"""
        entity.complete()
        self.w(u'<xcede:item ID="%s">\n' % entity.identifier)
        self.w(u'<xcede:itemText><xcede:textLabel location="leadText" value="%s"/></xcede:itemText>\n'
               % entity.text)
        self.w(u'<xcede:itemChoice itemCode="1" itemValue="%s"/>\n' % entity.possible_answers)
        self.w(u'</xcede:item>\n')


class QuestionnaireRunXcedeItemView(XMLItemView):
    __select__ = XMLItemView.__select__ & is_instance('QuestionnaireRun')
    __regid__ = 'xcede-item'
    templatable = False

    def entity_call(self, entity):
        """element as an item for an xml feed"""
        entity.complete()
        assessment = entity.reverse_generates[0] if entity.reverse_generates else None
        questionnaire = entity.instance_of[0]
        # XXX studyID/visitID -> Assessment
        self.w(u'<xcede:data xsi:type="xcede:assessment_t" subjectID="%s">\n'
               % xml_escape(entity.concerns[0].identifier))
        self.w(u'<xcede:name>%s</xcede:name>\n' % xml_escape(questionnaire.name))
        self.w(u'<xcede:dataInstance validated="%s">\n' % entity.valid)
        if questionnaire.note:
            self.w(u'<xcede:assessmentInfo><xcede:description>%s</xcede:description></xcede:assessmentInfo>'
                   % xml_escape(questionnaire.note))
        rset = self._cw.execute('Any AV, QI, QP ORDERBY QP ASC WHERE A is Answer, A question Q, '
                                'Q position QP, Q identifier QI, A value AV, A questionnaire_run QR, QR eid %(e)s',
                                {'e': entity.eid})
        for value, qid, qp in rset:
            self.w(u'<xcede:assessmentItem ID="%s"><xcede:value>%s</xcede:value></xcede:assessmentItem>\n'
                  % (xml_escape(qid), value))
        self.w(u'</xcede:dataInstance>\n')
        self.w(u'</xcede:data>\n')
        # Questionnaire
        # XXX Should be unique in the document
        ## questionnaire.view('xcede-item', w=self.w)


###############################################################################
### NEUROIMAGING ##############################################################
###############################################################################
class ScanXcedeItemView(XMLItemView):
    __select__ = XMLItemView.__select__ & is_instance('Scan')
    __regid__ = 'xcede-item'
    templatable = False

    def entity_call(self, entity):
        """element as an item for an xml feed"""
        entity.complete()
        assessment = entity.reverse_generates[0] if entity.reverse_generates else None
        # XXX studyID/visitID -> Assessment
        self.w(u'<acquisition ID="%(id)s" projectID="%(p)s" subjectID="%(s)s" '
               'visitID="%(a)s" studyID="%(a)s" episodeID="%(la)s"/>\n'
               % {'id': xml_escape(entity.cwuri),
                  'p': xml_escape(entity.related_study[0].name),
                  's': xml_escape(entity.concerns[0].identifier),
                  'a': xml_escape(assessment.cwuri) if assessment else None,
                  'la': xml_escape(entity.label)})
        if entity.has_data:
            entity.has_data[0].view('xcede-item', w=self.w)


class MRIDataXcedeItemView(XMLItemView):
    __select__ = XMLItemView.__select__ & is_instance('MRIData')
    __regid__ = 'xcede-item'
    templatable = False

    def entity_call(self, entity):
        """element as an item for an xml feed"""
        entity.complete()
        scan = entity.reverse_has_data[0]
        self.w(u'<dataResource xsi:type="dimensionedBinaryDataResource_t">\n')
        self.w(u'<uri offset="0" size="262144">%s</uri>\n' % xml_escape(scan.full_filepath))
        ##self.w(u'<elementType>int32</elementType>')
        ##self.w(u'<byteOrder>msbfirst</byteOrder>')
        self.w(u'<elementType>float32</elementType>')
        self.w(u'<compression>%s</compression>' % xml_escape(scan.format))
        if entity.shape_x:
            self.w(u'<dimension label="x">\n')
            self.w(u'<size>%s</size>' % entity.shape_x)
            self.w(u'<spacing>%s</spacing>' % entity.voxel_res_x)
            # XXX harc-coded ?
            self.w(u'<direction>1 0 0</direction>\n')
            self.w(u'<units>mm</units>\n')
            self.w(u'</dimension>\n')
        if entity.shape_y:
            self.w(u'<dimension label="y">\n')
            self.w(u'<size>%s</size>' % entity.shape_y)
            self.w(u'<spacing>%s</spacing>' % entity.voxel_res_y)
            # XXX harc-coded ?
            self.w(u'<direction>0 1 0</direction>\n')
            self.w(u'<units>mm</units>\n')
            self.w(u'</dimension>\n')
        if entity.shape_z:
            self.w(u'<dimension label="y">\n')
            self.w(u'<size>%s</size>' % entity.shape_z)
            self.w(u'<spacing>%s</spacing>' % entity.voxel_res_z)
            # XXX harc-coded ?
            self.w(u'<direction>0 0 1</direction>\n')
            self.w(u'<units>mm</units>\n')
            self.w(u'</dimension>\n')
        self.w(u'</dataResource>\n')


###############################################################################
### GENOMIC ###################################################################
###############################################################################
class GenomicMeasureXcedeItemView(XMLItemView):
    __select__ = XMLItemView.__select__ & is_instance('GenomicMeasure')
    __regid__ = 'xcede-item'
    templatable = False

    def entity_call(self, entity):
        """element as an item for an xml feed"""
        entity.complete()
        assessment = entity.reverse_generates[0] if entity.reverse_generates else None
        # XXX studyID/visitID -> Assessment
        self.w(u'<acquisition ID="%(id)s" projectID="%(p)s" subjectID="%(s)s" '
               'visitID="%(a)s" studyID="%(a)s" episodeID="%(la)s"/>\n'
               % {'id': xml_escape(entity.cwuri),
                  'p': xml_escape(entity.related_study[0].name),
                  's': xml_escape(entity.concerns[0].identifier),
                  'a': xml_escape(assessment.cwuri) if assessment else None,
                  'la': xml_escape(entity.type)})



# copyright 2012 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

""" Schema test """

from cubicweb.devtools.testlib import CubicWebTC


class BrainomicsEntitiesTC(CubicWebTC):

    def _create_subject(self):
        req = self.request()
        study = req.create_entity('Study', name=u'Test study')
        subj = req.create_entity('Subject', identifier=u'test_subj')
        sdef = req.create_entity('ScoreDefinition', name=u'Test score def', type=u'string')
        sval = req.create_entity('ScoreValue', definition=sdef, text=u'test score val')
        return study, subj, sdef, sval

    def _create_measure(self, study, subj):
        req = self.request()
        center = req.create_entity('Center', identifier=u'Test center ID',
                                   name=u'Test center name')
        assessment = req.create_entity('Assessment',
                                       related_study=study,
                                       reverse_holds=center,
                                       reverse_concerned_by=subj)
        questionnaire = req.create_entity('Questionnaire', name=u'Test quiz',
                                          identifier=u'test_quiz_id',
                                          type=u'test')
        qrun = req.create_entity('QuestionnaireRun',
                                 user_ident=u'test_qrun_uident',
                                 instance_of=questionnaire,
                                 reverse_generates=assessment,
                                 concerns=subj,
                                 related_study=study)
        return qrun

    def _create_sample(self, study, subj):
        req = self.request()
        biopsy = req.create_entity('Biopsy',
                                   related_study=study,
                                   reverse_related_samples=subj)
        return biopsy

    def test_score_value_subject_1(self):
        study, subj, sdef, sval = self._create_subject()
        subj.cw_set(related_infos=sval)
        self.commit()
        self.assertEqual(sval.subject, subj)

    def test_score_value_subject_2(self):
        study, subj, sdef, sval = self._create_subject()
        qrun = self._create_measure(study, subj)
        sval.cw_set(measure=qrun)
        self.commit()
        self.assertEqual(sval.subject, subj)

    def test_score_value_subject_3(self):
        study, subj, sdef, sval = self._create_subject()
        biopsy = self._create_sample(study, subj)
        sval.cw_set(score_of_sample=biopsy)
        self.commit()
        self.assertEqual(sval.subject, subj)


if __name__ == '__main__':
    from logilab.common.testlib import unittest_main
    unittest_main()



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# copyright 2013 CEA (Saclay, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-brainomics schema"""
from itertools import chain

from cubicweb.schemas.base import CWUser
from yams.buildobjs import RelationDefinition, SubjectRelation, String

from cubes.questionnaire.schema import QuestionnaireRun, Questionnaire, Question
from cubes.neuroimaging.schema import Scan, AnatomicalRegion
from cubes.genomics.schema import GenomicMeasure, ColumnRef, Gene
from cubes.medicalexp.schema  import (Assessment, ProcessingRun, ScoreDefinition, ScoreValue,
                                      Diagnostic, TechnicalAnalysis, Subject)
from cubes.clinipath.schema import Biopsy, BiopsySample, CellCulture, BloodSample, IHCMeasure


# Diagnostic/TechnicalAnalysis
for etype in ('QuestionnaireRun', 'Scan', 'GenomicMeasure'):
    Diagnostic.add_relation(SubjectRelation(etype, cardinality='**'), name='based_on')
    # No composite for TechnicalAnalysis (for container)
    TechnicalAnalysis.add_relation(SubjectRelation(etype, cardinality='**'), name='performed_on')
for etype in ('Biopsy', 'BiopsySample', 'CellCulture', 'BloodSample'):
    Diagnostic.add_relation(SubjectRelation(etype, cardinality='**'), name='based_on')
    # No composite for TechnicalAnalysis (for container)
    TechnicalAnalysis.add_relation(SubjectRelation(etype, cardinality='**'), name='analysis_of_sample')
for etype in ('QuestionnaireRun', 'Scan', 'GenomicMeasure', 'IHCMeasure'):
    ProcessingRun.add_relation(SubjectRelation(etype, cardinality='**'), name='inputs')
    ProcessingRun.add_relation(SubjectRelation(etype, cardinality='**'), name='outputs')
    Assessment.add_relation(SubjectRelation(etype, cardinality='**',
                                            composite='subject'), name='uses')
    Assessment.add_relation(SubjectRelation(etype, cardinality='**',
                                            composite='subject'), name='generates')
# Score -> Measure
for etype in ('QuestionnaireRun', 'Scan', 'GenomicMeasure', 'IHCMeasure'):
    ScoreValue.add_relation(SubjectRelation(etype, cardinality='?*', inlined=True,
                                            composite='object'), name='measure')
for etype in ('Biopsy', 'BiopsySample', 'CellCulture', 'BloodSample'):
    ScoreValue.add_relation(SubjectRelation(etype, cardinality='?*', inlined=True), name='score_of_sample')
# Measure -> Subject
for etype_class in (QuestionnaireRun, Scan, GenomicMeasure, IHCMeasure):
    etype_class.add_relation(SubjectRelation(('Subject', 'SubjectGroup'),
                                             cardinality='1*', inlined=True), name='concerns')
    etype_class.add_relation(SubjectRelation('Study', cardinality='1*', inlined=True,
                                             composite='object'), name='related_study')
    etype_class.add_relation(SubjectRelation('Study', cardinality='**'), name='other_studies')
    etype_class.add_relation(SubjectRelation('Device', cardinality='?*',
                                             inlined=True), name='uses_device')
# Some questionnaire run could have specific results files (subject dependant)
Questionnaire.add_relation(SubjectRelation(('File', 'FileSet', 'ExternalFile'),
                                           cardinality='**', composite='subject'),
                           name='configuration_files')
Questionnaire.add_relation(SubjectRelation('ScoreDefinition', cardinality='*?'),
                           name='definitions')
ScoreDefinition.add_relation(SubjectRelation('Question', cardinality='**'),
                             name='used_by')
AnatomicalRegion.add_relation(SubjectRelation('ScoreValue', cardinality='**'),
                              name='concerned_by')
ColumnRef.add_relation(SubjectRelation('Assessment', cardinality='1*', inlined=True,
                                       composite='object'), name='assessment')
# Measure -> Files
for etype_class in (QuestionnaireRun, Scan, GenomicMeasure, IHCMeasure):
    etype_class.add_relation(SubjectRelation(('File', 'FileSet', 'ExternalFile'),
                                             cardinality='**', composite='subject'),
                             name='results_files')
    etype_class.add_relation(SubjectRelation(('File', 'FileSet', 'ExternalFile'),
                                             cardinality='**', composite='subject'),
                             name='configuration_files')
# Extra fields for CWUser
CWUser.add_relation(String(maxsize=512, fulltextindexed=True), name='affiliation')
CWUser.add_relation(String(maxsize=512, fulltextindexed=True), name='extra_infos')
# Genomic
Gene.add_relation(SubjectRelation('Study', cardinality='**'), name=u'relevant_gene_for')
# Clinipath
Subject.add_relation(SubjectRelation(('Biopsy', 'BiopsySample', 'CellCulture', 'BloodSample'),
                                     cardinality='?*', composite='subject'),
                     name='related_samples')
GenomicMeasure.add_relation(SubjectRelation(('Biopsy', 'BiopsySample', 'CellCulture', 'BloodSample'),
                                            cardinality='**'),
                            name='measured_on_sample')
for etype_class in (Biopsy, BiopsySample, CellCulture, BloodSample):
    etype_class.add_relation(SubjectRelation('MedicalTechnique', cardinality='?*',
                                             inlined=True),
                             name='technique_type')
    etype_class.add_relation(SubjectRelation('BodyLocation', cardinality='?*'),
                             name='from_location')
    etype_class.add_relation(SubjectRelation('Study', cardinality='1*', inlined=True),
                             name='related_study')
    etype_class.add_relation(SubjectRelation('Study', cardinality='**'),
                             name='other_studies')
    etype_class.add_relation(SubjectRelation('Device', cardinality='?*', inlined=True),
                             name='uses_device')


# Comments for entities
class comments(RelationDefinition):
    subject = 'Comment'
    object = ('Scan', 'GenomicMeasure', 'QuestionnaireRun', 'IHCMeasure')
    composite='object'


# Links to "Wikis" / Cards
class wiki(RelationDefinition):
    subject = ('Question', 'Questionnaire', 'Study', 'ScoreDefinition')
    object = 'Card'
    cardinality = '**'
    composite='subject'



# -*- coding: utf-8 -*-
# copyright 2014 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# copyright 2014 CEA (Saclay, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

from cubicweb.web.views.csvexport import CSVRsetView
from cubicweb.predicates import is_instance, one_line_rset


class GenomicMeasureCSVView(CSVRsetView):
    """CSV view for GenomicMeasure"""
    __select__ = (CSVRsetView.__select__ & is_instance('GenomicMeasure')
                  & ~one_line_rset())

    def call(self):
        req = self._cw
        rows = [('"subject identifier"', '"subject admission number"',
                 '"study"', '"type"', '"results_files"')]
        for entity in self.cw_rset.entities():
            study = entity.related_study[0]
            subject = entity.concerns[0]
            admissions = [a for a in subject.reverse_admission_of
                          if a.admission_in[0].eid == study.eid]
            subj_adm_no = (admissions[0].subject_admission_number
                           if admissions else self._cw._('No admission number'))
            rows.append([subject.identifier,
                         subj_adm_no,
                         study.name,
                         entity.type,
                         '; '.join(f.data_name for f in entity.results_files)])
        writer = self.csvwriter()
        writer.writerows(rows)



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# copyright 2013 CEA (Saclay, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

import os
import os.path as osp
import tempfile
from contextlib import closing
from zipfile import ZipFile, ZIP_DEFLATED

from cubicweb.predicates import is_instance
from cubicweb.view import EntityView


CSV_DOWNLOADABLE = ('Subject', 'Scan', 'ScoreDefinition', 'QuestionnaireRun',
                    'Answer', 'GenomicMeasure', 'Snp', 'GenericTestRun')
XCEDE_DOWNLOADABLE = ('Subject', 'Scan', 'GenomicMeasure', 'QuestionnaireRun', 'Questionnaire')
JSON_DOWNLOADABLE = ('Subject',)
ZIP_DOWNLOADABLE = ('Scan', 'GenomicMeasure')
ALL_DOWNLOADABLE = set(CSV_DOWNLOADABLE + XCEDE_DOWNLOADABLE + ZIP_DOWNLOADABLE)


###############################################################################
### ZIP VIEWS #################################################################
###############################################################################
class DataZipView(EntityView):
    """Abstract base class for the zip view"""
    __regid__ = 'data-zip'
    __select__ = EntityView.__select__ & is_instance(*ZIP_DOWNLOADABLE)
    templatable = False
    binary = True
    archive_name = 'brainomics_data.zip'

    def _zipfiles(self, filepaths):
        """generates a zip archive from `filepaths`

        :param filepaths: is a set containing tuples with relative file paths
                          as they appear in the archive, and CubicWeb file
                          objects, whose content is put in the archive.

        :return: a couple fileobj, filepath where `fileobj` is a standard python
                 open file objet on the generated archive, and `filepath` is an
                 absolute file path to this archive on the filesystem
        """
        fd, archive_filepath = tempfile.mkstemp()
        try:
            noext_archivename = osp.splitext(self.archive_name)[0]
            with closing(ZipFile(archive_filepath, "w", ZIP_DEFLATED, True)) as zip:
                for filename, gmes_type, subjs, _fileobj in filepaths:
                    for subj in subjs:
                        zip.writestr('%s/%s/%s/%s' % (noext_archivename, gmes_type,
                                                      subj, filename),
                                     _fileobj.read())
            fileobj = os.fdopen(fd)
            fileobj.seek(0)
            return fileobj, archive_filepath
        except:
            os.close(fd)
            os.unlink(archive_filepath)
            raise

    def set_request_content_type(self):
        self._cw.set_content_type('application/zip', filename=self.archive_name)

    def make_entity_labels(self, entity, prefix):
        return ('{0}_{1}'.format(prefix, entity.concerns[0].identifier),)

    def call(self):
        if not self.cw_rset:
            return
        filepaths = set()
        prefix = self._cw._('Subject')
        for entity in self.cw_rset.entities():
            subj = self.make_entity_labels(entity, prefix)
            for _file in entity.results_files:
                if _file.__regid__ == 'FileSet':
                    for _f in _file.file_entries:
                        filepaths.add((_file.dc_title(), entity.dc_title(), subj, _f))
                else:
                        filepaths.add((_file.dc_title(), entity.dc_title(), subj, _file))
        fileobj, archive_filepath = self._zipfiles(list(filepaths))
        try:
            self.w(fileobj.read())
        finally:
            fileobj.close()
            os.unlink(archive_filepath)



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# copyright 2013 CEA (Saclay, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
import json

from cubicweb.utils import make_uid

from cubicweb.web.views import tableview as cw_tableview


"""
This module provides SlickGrid table layout and a SlickgridMixin that
can be used with a TableView to have a SlickGrid representation of the data.

It also provides two implementations of this Mixin:

 * SlickGridRsetTableView

 * SlickGridEntityTableView


These SlickGrid views may be used with:

 * a grid_options attribute (e.g. fro navigation);

 * a default_column_options (e.g. sortable, width, ...);

 * a slickgrid_columns_options that allows to define specific
   options for a given column. The name should be the name
   of a column renderer;

 * a default_formatter attribute;

For each column renderer, without further specifications, a default
column option is built
"""


###############################################################################
### TABLE_SLICKGRID VIEW ######################################################
###############################################################################

class SlickGridTableLayout(cw_tableview.TableLayout):
    __regid__ = 'slickgrid-layout'
    display_filter = 'top'
    hide_filter = False

    def _add_resources(self):
        """ Add the resources (CSS, JS) for slickgrid """
        req = self._cw
        req.add_css('SlickGrid/slick.grid.css')
        req.add_css('cubes.brainomics.slickgrid.css')
        req.add_js('jquery.event.drag-2.2.js')
        req.add_js('SlickGrid/slick.dataview.js')
        req.add_js('SlickGrid/slick.core.js')
        req.add_js('SlickGrid/slick.grid.js')
        req.add_js('SlickGrid/slick.formatters.js')
        req.add_js('cubes.brainomics.slickgrid.js')

    def create_container(self, **options):
        return (u'<div class="slickgrid" id="%(divid)s" '
                u'style="width: %(width)s; height: %(height)s;">'
                u'</div>') % options


    def render_table(self, w, actions, paginate, **kwargs):
        """ Render the slickgrid object """
        self._add_resources()
        self.data = []
        view = self.view
        container_options = view.container_options
        divid = view.domid
        container_options['divid'] = divid
        columns = self.build_grid_column_options()
        data = json.dumps(self.init_data(view, columns))
        grid_options = json.dumps(view.grid_options)
        self._cw.add_onload('cwSlickGrid("#%s", %s, %s, %s)'
                       % (divid,  data,
                          json.dumps(columns),
                          grid_options))
        if actions and self.display_actions == 'top':
            self.render_actions(w, actions)
        w(self.create_container(**container_options))
        if actions and self.display_actions == 'bottom':
            self.render_actions(w, actions)

    def init_data(self, view, columns):
        _columns = set(data['field'] for data in columns)
        colrenderers = view.build_column_renderers()
        rows = []
        for rownum in xrange(self.view.table_size):
            _row = {'id': rownum}
            for colnum, renderer in enumerate(colrenderers):
                stream = []
                w = stream.append
                renderer.render_cell(w, rownum)
                _row[renderer.colid] = stream[-1]
            rows.append(_row)
        return rows

    def build_grid_column_options(self):
        _ = self._cw._
        existing_columns = dict([(c['name'], c)
                                 for c in self.view.slickgrid_columns_options])
        columns = []
        for idx, renderer in enumerate(self.view.build_column_renderers()):
            column_option = existing_columns.get(renderer.colid,
                                                 {'name': renderer.colid})
            columns.append(self.view.build_column_option(idx, column_option))
        return columns


class SlickGridMixIn(object):
    layout_id = 'slickgrid-layout'
    paginable = False
    # slickgrid options
    grid_options = {'enableCellNavigation':True,}
    # columns slickgrid related options
    default_column_options = {'sortable':True}
    # Define in concrete views
    slickgrid_columns_options = ()
    default_formatter = 'HTML'

    def entity(self, rownum):
        """Return the table's main entity"""
        return self.cw_rset.get_entity(rownum, self.cw_col or 0)

    @property
    def container_options(self):
        return {'width':'100%%',
                'height':'400px'}

    def build_column_option(self, idx, column_option):
        column = column_option.copy()
        column['formatter'] = self.default_formatter
        name = _(column_option['name'])
        column['name'] = self._cw._(name)
        if 'id' not in column_option:
            column['id'] = '_c%i' % idx
        if 'field' not in column_option:
            column['field'] = name
        column.update(self.default_column_options)
        return column

    def build_grid_column_options(self):
        _ = self._cw._
        columns = []
        for idx, column_option in enumerate(self.slickgrid_columns_options):
            columns.append(self.build_column_option(idx, column_option))
        return columns


class  SlickGridRsetTableView(SlickGridMixIn, cw_tableview.RsetTableView):
    __regid__ = 'slickgrid-table'


class  SlickGridEntityTableView(SlickGridMixIn, cw_tableview.EntityTableView):
    __abstract__ = True



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# copyright 2013 CEA (Saclay, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
from collections import defaultdict
from logilab.mtconverter import xml_escape

from cubicweb.predicates import is_instance
from cubicweb.view import EntityView
from cubicweb.web.views.baseviews import ListView, SameETypeListView


class BrainomicsListView(ListView):

    def call(self, klass=None, title=None, subvid=None, listid=None, **kwargs):
        super(BrainomicsListView, self).call(klass=klass or 'list-unstyled',
                                             title=title, subvid=subvid,
                                             listid=listid, **kwargs)


class BrainomicsSameETypeListView(SameETypeListView):

    def call(self, klass='list-striped', **kwargs):
        super(BrainomicsSameETypeListView, self).call(klass='list-unstyled', **kwargs)


class BrainomicsAdmissionView(EntityView):
    __regid__ = 'admission-view'
    __select__ = EntityView.__select__ & is_instance('Admission')

    def cell_call(self, row, col):
        entity = self.cw_rset.get_entity(row, col)
        study = entity.admission_in
        w = self.w
        w(u'<ul class="list-group">')
        w(u'<div class="panel panel-default">')
        w(u'<div class="panel-heading">%s (%s %s)</div>' % (entity.dc_title(),
                                                            self._cw._('see study'),
                                                            entity.admission_in[0].view('incontext')))
        w(u'<div class="panel-body">')
        w(u'%s: %s %s' % (self._cw._('Subject age'), entity.subject_age, self._cw._('years')))
        w(u'<ul>')
        for step in entity.reverse_step_of:
            w(u'<li>%s</li>' % step.dc_title())
        w(u'</ul>')
        if entity.admission_scores:
            w(self._cw._('Scores at admission'))
            w(u'<ul>')
            for score in entity.admission_scores:
                w(u'<li>%s</li>' % score.dc_title())
            w(u'</ul>')
        w(u'</div>')
        w(u'</div>')


class BrainomicsCardDocumentation(EntityView):
    __regid__ = 'documentation_card'
    __select__ = EntityView.__select__ & is_instance('Card')

    def call(self, rset=None, **kwargs):
        rset = rset or self.cw_rset
        w = self.w
        w(u'<div>')
        for entity in rset.entities():
            w(u'<h2>%s</h2>' % xml_escape(entity.title))
            if entity.synopsis:
                w(u'<blockquote><p>%s</p></blockquote>' % xml_escape(entity.synopsis))
            if entity.content:
                w(entity.content)
        w(u'</div>')


###############################################################################
### RESOURCES VIEW ############################################################
###############################################################################
class BrainomicsResultFile(EntityView):
    __regid__ = 'results-view'
    __select__ = EntityView.__select__ & is_instance('Questionnaire', 'QuestionnaireRun',
                                                     'GenomicMeasure', 'Scan')

    def entity_call(self, entity):
        # Display results file
        w = self.w
        rset = self._cw.execute('Any F WHERE X results_file F, X eid %(e)s',
                                {'e': entity.eid})
        if rset:
            w(u'<h3>%s</h3>' % self._cw._('Results file'))
            self.wview('list', rset=rset)


class AnatomicPathologyReportView(EntityView):
    __regid__ = 'clinipath-reports'
    __select__ = EntityView.__select__ & is_instance('AnatomicPathologyReport')

    def call(self, rset=None):
        w = self.w
        w(u'<div class="panel-group" id="biopsy-reports">')
        for entity in self.cw_rset.entities():
            self.entity_call(entity)
        w(u'</div>')
        # Js
        self._cw.add_onload('$(".collapse").collapse()')

    def entity_call(self, entity):
        w = self.w
        w(u'<div class="panel panel-default">')
        w(u'<div class="panel-heading">')
        w(u'<h4 class="panel-title">')
        w(u'<a data-toggle="collapse" data-parent="#biopsy-reports" href="#report-%s">%s</a>'
          % (entity.eid, entity.formatted_title))
        w(u'</h4>')
        w(u'</div>')
        w(u'<div id="report-%s" class="panel-collapse collapse in">' % entity.eid)
        w(u'<div class="panel-body">%s</div>' % entity.formatted_content)
        w(u'</div>')
        w(u'</div>')


class BrainomicsAggregatedView(EntityView):
    __regid__ = 'aggregated-view'

    def call(self, rset=None):
        rset = rset or self.cw_rset
        row_types = list(set(row[0] for row in rset.description))
        if len(row_types) > 1 or not row_types:
            raise RQLException('Wrong result set query: %s' % rset.rql)
        row_type = row_types[0]
        table = defaultdict(dict)
        for row in rset:
            table.setdefault(row[0], {})
            table[row[0]].update({row[1]: row[2]})
        return self._html_table_from_data(table, row_type)

    def _html_table_from_data(self, table_data, row_type):
        """Build html table out of a Python dictionary with eids and values.

           `table_data` is a dictionary with one key for each table row.
           The value of each such key is a dictionary, with one key for each column.
           The value of such a key is the value one wishes to put in the table cell.

           `row_type` is a string holding the yams entity type of the entities
           which occupy the first cell on each row.
        """
        # header first
        self.w(u'<table><tr><th>%s</th>' % xml_escape(self._cw._(row_type)))
        # Empty td first to leave room for first column
        header_content = ''.join(u'<th><a href="%s">%s</a></th>'
                                 % (ent.absolute_url(), ent.dc_title())
                                 for ent in map(self._cw.entity_from_eid,
                                                set(kcol for krow in table_data
                                                    for kcol in table_data[krow])))
        self.w(header_content)
        self.w(u'</tr>')
        # now the rows
        for row in table_data:
            self.w(u'<tr>')
            # first row element:
            row_ent = self._cw.entity_from_eid(row)
            self.w(u'<td align="center"><a href="%s">%s</td>'
                   % (row_ent.absolute_url(), row_ent.dc_title()))
            row_content = ''.join(u'<td align="center">%s</td>'
                                  % table_data[row][col]
                                  for col in table_data[row])
            self.w(row_content)
            self.w(u'</tr>')
        self.w(u'</table>')


def registration_callback(vreg):
    vreg.register_all(globals().values(), __name__, (BrainomicsListView,
                                                     BrainomicsSameETypeListView))
    vreg.register_and_replace(BrainomicsListView, ListView)
    vreg.register_and_replace(BrainomicsSameETypeListView, SameETypeListView)




# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# copyright 2013 CEA (Saclay, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-brainomics views/forms/actions/components for web ui"""

from cubicweb.web import formfields as ff, Redirect, ValidationError
import cubicweb.web.formwidgets as fwdgs
from cubicweb.view import AnyRsetView
from cubicweb.web.views.basecontrollers import ViewController
from cubicweb.web.views import forms
from cubicweb.web.action import Action


###############################################################################
### BROWSER LIST ##############################################################
###############################################################################
BROWSABLE_ENTITIES = {'Subject': 'identifier'}


class BrainomicsBrowseEntityList(AnyRsetView):
    __regid__ = 'browse-list'

    def call(self):
        self.w(u'<div class="col-md-12">')
        self.w(u'<h2>%s</h2>' % self._cw._('Browse entities list'))
        self.w(u'<div class="well">%s</div>'
               % self._cw._('Browse for a list of entities using a CSV file. The CSV file should have '
                            'only one identifier by line.'))
        entities_form = self._cw.vreg['forms'].select('browse-list-form', self._cw)
        entities_form.render(w=self.w)
        self.w(u'</div>')


class BrainomicsBrowseEntityListForm(forms.FieldsForm):
    __regid__ = 'browse-list-form'
    form_buttons = [fwdgs.SubmitButton(label=_('Validate'), attrs={"class": "btn btn-primary"})]

    @property
    def action(self):
        return self._cw.build_url('browse-list-controller')

    filename = ff.FileField(name='__file',
                            label=_('Indentifiers file to import'),
                            order=1,)

    etype = ff.StringField(name='__etype',
                           label=_('Entity type'),
                           required=True,
                           help=_('Entity type'),
                           order = 2,
                           choices = BROWSABLE_ENTITIES.keys())


class BrainomicsBrowseEntityListController(ViewController):
    __regid__ = 'browse-list-controller'

    def _read_uuid_file(self, filename):
        try:
            fdesc = filename[1]
        except IOError as ioerr:
            raise ValidationError(None, 'File not found')
        else:
            uuids = ["'" + uuid.strip() + "'" for uuid in fdesc.readlines() if uuid.strip()]
        return uuids

    def publish(self, rset=None):
        filename = self._cw.form['__file']
        etype = self._cw.form['__etype']
        uuids = self._read_uuid_file(filename)
        if uuids:
            rql = ('Any X WHERE X is %(etype)s, X %(ident)s IN (%(uuids)s)'
                   % {'uuids': ', '.join(uuids), 'etype': etype,
                      'ident': BROWSABLE_ENTITIES[etype]})
            raise Redirect(self._cw.build_url(vid='list', rql=rql))
        raise Redirect(self._cw.base_url())


class BrainomicsBrowseEntityListAction(Action):
    __regid__ = 'browse-list-action'
    category = 'browse-links'
    title = _('Browse from list')

    def url(self):
        return self._cw.build_url(vid='browse-list')



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# copyright 2013 CEA (Saclay, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

from cubicweb.utils import json_dumps, js_dumps, JSString
from cubicweb.predicates import multi_columns_rset

from cubes.jqplot.views import JQPlotSimpleView


################################################################################
### JQPLOT RSET VIEW ###########################################################
################################################################################
class BrainomicsRsetJqplot(JQPlotSimpleView):
    __regid__ = 'jqplot-2col'
    __select__ = multi_columns_rset(2)

    def call(self, rset=None, tab_id=None, jsbind=None, renderer=None, options=None,
             divid=None, legend=None, colors=None,
             width=450, height=300, displayfilter=False, mainvar=None, title=None,
             displayactions=False, actions=None):
        # Refefine the call() function to allow to pass an rset
        if self._cw.ie_browser():
            self._cw.add_js('excanvas.js')
        self._cw.add_js(('jquery.jqplot.js', 'cubes.jqplot.js', 'cubes.jqplot.ext.js'))
        self._cw.add_css('jquery.jqplot.min.css')
        data, ticks = self.get_data(rset)
        if legend is None:
            legend = self.default_legend
        if divid is None:
            divid = u'figure%s' % self._cw.varmaker.next()
        if renderer is None:
            renderer = self.default_renderer
        serie_options = {'renderer': self.renderer(self.renderers, renderer)}
        if options is None:
            options = self.default_options
        serie_options['rendererOptions']= options
        options = {'series': [serie_options],
                   'legend': legend,
                   'title': title,
                   }
        if ticks:
            self._cw.html_headers.add_onload('ticks = %s' % json_dumps(ticks))
            options['axes'] = {'xaxis': {'ticks': JSString('ticks'),
                                         'renderer': JSString('$.jqplot.CategoryAxisRenderer')}}
            self._cw.add_js('plugins/jqplot.categoryAxisRenderer.min.js')
        if colors is not None:
            options['seriesColors'] = colors
        self.set_custom_options(options)
        # Allow an onload on tab/pill show
        js_onload  = self.onload % {'id': divid, 'data': json_dumps([data]),
                                    'options': js_dumps(options)}
        if not tab_id:
            self._cw.html_headers.add_onload(js_onload)
        else:
            self._cw.html_headers.add_onload("""$('a[data-toggle="%s"]').on('shown.bs.tab',
            function (e) {%s})""" % (tab_id, js_onload))
        self.div_holder(divid, width, height)
        if displayfilter and not 'fromformfilter' in self._cw.form:
            self.form_filter(divid, mainvar)
        if displayactions and not 'fromformfilter' in self._cw.form:
            self.display_actions(divid, actions)
        # Add a js bind function
        if jsbind:
            self._cw.html_headers.add_onload("""$('#%s').bind('jqplotDataClick',%s)"""
                                             % (divid, jsbind))

    def get_data(self, rset=None):
        rset = rset or self.cw_rset
        if isinstance(rset[0][0], basestring):
            ticks = list(set([r[0] for r in rset if r[0] is not None and r[1] is not None]))
            data = [[ticks.index(r[0]), r[1]] for r in rset if r[0] is not None and r[1] is not None]
            return [d[1] for d in sorted(data, key=lambda x: x[0])], ticks
        else:
            return [[r[0], r[1]] for r in rset if r[0] is not None and r[1] is not None], None
